library iogunb;

uses
  SysUtils,
  StrUtils,
  Classes,
  Math,
  Windows;

type TIntList = class
private
  _array : array of Integer;
  _count : Integer;
public
  constructor Create;
  destructor Destroy; override;
  property Count : Integer read _count;
  procedure Add ( value : Integer );
  procedure Parse ( text : string );
  function Union ( other : TIntList ) : TIntList;
  function Intersection ( other : TIntList ) : TIntList;
  function Exclusion ( other : TIntList ) : TIntList;
  function Has ( value : Integer ) : Boolean;
  function AsString : string;
  function AsCompressedString : string;
end;

constructor TIntList.Create;
begin
  SetLength ( _array, 0 );
  _count := 0;
end;

destructor TIntList.Destroy;
begin
  _array := nil;
  inherited Destroy;
end;

procedure TIntList.Add(value:Integer);
var len: Integer;
begin
  if not Has (value) then
  begin
    len := _count;
    Inc(_count);
    SetLength ( _array, _count );
    _array[len] := value;
  end;
end;

procedure TIntList.Parse ( text : string );
var c: char;
  s : string;
  i, first, second : Integer;
  hasFirst, hasSecond : Boolean;

  procedure DoAdd;
  var j: Integer;
  begin
    if hasSecond then second := StrToInt(s)
    else first := StrToInt(s);
    if hasFirst then
    begin
      if hasSecond then
        for j := first to second do
          Add ( j )
      else
        Add ( first );
    end;
    hasFirst := False;
    hasSecond := False;
    s := '';
  end;

begin
  hasFirst := False;
  hasSecond := False;
  text := AnsiReplaceStr ( text, ' ', '' );
  for i := 1 to Length (text) do
  begin
    c := text[i];
    case c of
      '0'..'9':
        begin
          s := s + c;
          hasFirst := True;
        end;
      '-':
        begin
          first := StrToInt(s);
          s := '';
          hasSecond := True;
        end;
      ',', '/':
        begin
          DoAdd;
        end;
    else
      // Error!
    end;
  end;
  DoAdd;
end;

function TIntList.Union ( other : TIntList ) : TIntList;
var r: TIntList;
  i, j : Integer;
begin
  r := TIntList.Create;

  for i := Low (_array) to High (_array) do
    r.Add (_array[i]);
  for i := Low (other._array) to High (other._array) do
  begin
    j := other._array[i];
    r.Add (j);
  end;

  Result := r;
end;

function TIntList.Intersection ( other : TIntList ) : TIntList;
var r: TIntList;
  i, j : Integer;
begin
  r := TIntList.Create;

  for i := Low (_array) to High (_array) do
  begin
    j := _array[i];
    if other.Has(j) then r.Add(j);
  end;

  Result := r;
end;

function TIntList.Exclusion ( other : TIntList ) : TIntList;
var r: TIntList;
  i, j : Integer;
begin
  r := TIntList.Create;

  for i := Low (_array) to High (_array) do
  begin
    j := _array[i];
    if not other.Has(j) then r.Add(j);
  end;

  Result := r;
end;

function TIntList.Has (value:Integer):Boolean;
var i : Integer;
begin
  Result := False;
  for i := Low (_array) to High (_array) do
    if (_array[i] = value ) then
    begin
      Result := True;
      Break;
    end;
end;

function TIntList.AsString : string;
var text : string;
  i : Integer;
begin
  for i := Low (_array) to High (_array) do
    text := text + ',' + IntToStr(_array[i]);
  if Length(text) <> 0 then text := RightStr ( text, Length(text) - 1 );
  Result := text;
end;

function TIntList.AsCompressedString : string;
var text : string;
  i, j, prev, last : Integer;

  procedure AddRange;
  var s : string;
  begin
    if (prev = last) then s := IntToStr (prev)
    else if (prev = (last - 1)) then s := IntToStr (prev) + ',' + IntToStr (last) 
         else s := IntToStr (prev) + '-' + IntToStr(last);
    if Length(text) <> 0 then s := ',' + s;
    text := text + s;
  end;

begin
  text := '';
  Result := '';
  if Length(_array) = 0 then Exit;

  prev := _array[Low (_array)];
  last := prev;

  for i := Low (_array) + 1 to High (_array) do
  begin
    j := _array[i];
    if j <> (last + 1) then
    begin
      AddRange;
      prev := j;
    end;
    last := j;
  end;

  AddRange;

  Result := text;
end;

function GetInput ( txt : PChar ): string;
begin
  Result := string ( txt );
end;

procedure SetOutput ( txt, buf : PChar; size : integer );
var
  len : integer;
begin
  len := Min ( lstrlen ( txt ) + 1, size );
  lstrcpyn ( buf, txt, len );
end;

function SplitAnsiInput ( txt : string ): TStringList;
var
  strings : TStringList;
begin
  strings := TStringList.Create;
  ExtractStrings ( ['|'], [], PChar ( txt ), strings );
  Result := strings;
end;

procedure SplitTo ( s : string; var s1, s2 : string );
var sl : TStringList;
begin
  sl := SplitAnsiInput(s);
  s1 := sl[0];
  s2 := sl[1];
end;

//
// Sample:
// &unifor ('+8iogunb,union,1,2,3|3,4')
// result: 1-4
//

function Union ( buf1, buf2 : PChar; size: integer ): integer; stdcall;
var t, t1, t2, t3 : string;
  s1, s2, s3 : TIntList;
begin
  t := GetInput ( buf1 );
  SplitTo ( t, t1, t2 );
  s1 := TIntList.Create;
  s1.Parse(t1);
  s2 := TIntList.Create;
  s2.Parse(t2);
  s3 := s1.Union(s2);
  t3 := s3.AsCompressedString;
  SetOutput ( PChar (t3), buf2, size );
  s1.Destroy ();
  s2.Destroy ();
  s3.Destroy ();
  Result := 1;
end;

//
// Sample:
// &unifor ('+8iogunb,intersect,1,2,3|3,4')
// result: 3
//

function Intersection ( buf1, buf2 : PChar; size: integer ): integer; stdcall;
var t, t1, t2, t3 : string;
  s1, s2, s3 : TIntList;
begin
  t := GetInput ( buf1 );
  SplitTo ( t, t1, t2 );
  s1 := TIntList.Create;
  s1.Parse(t1);
  s2 := TIntList.Create;
  s2.Parse(t2);
  s3 := s1.Intersection(s2);
  t3 := s3.AsCompressedString;
  SetOutput ( PChar (t3), buf2, size );
  s1.Destroy ();
  s2.Destroy ();
  s3.Destroy ();
  Result := 1;
end;

//
// Sample:
// &unifor ('+8iogunb,exclude,1,2,3|3,4')
// result: 1,2
//

function Exclusion ( buf1, buf2 : PChar; size: integer ): integer; stdcall;
var t, t1, t2, t3 : string;
  s1, s2, s3 : TIntList;
begin
  t := GetInput ( buf1 );
  SplitTo ( t, t1, t2 );
  s1 := TIntList.Create;
  s1.Parse(t1);
  s2 := TIntList.Create;
  s2.Parse(t2);
  s3 := s1.Exclusion(s2);
  t3 := s3.AsCompressedString;
  SetOutput ( PChar (t3), buf2, size );
  s1.Destroy ();
  s2.Destroy ();
  s3.Destroy ();
  Result := 1;
end;

//
// Sample:
// &unifor ('+8iogunb,missing,1-100,105-200,203-360,201/202|1-365')
// result: 101-104,361-365
//

function Missing ( buf1, buf2 : PChar; size: integer ): integer; stdcall;
var t, t1, t2, t3 : string;
  s1, s2, s3 : TIntList;
begin
  t := GetInput ( buf1 );
  SplitTo ( t, t1, t2 );
  s1 := TIntList.Create;
  s1.Parse(t2);
  s2 := TIntList.Create;
  s2.Parse(t1);
  s3 := s1.Exclusion(s2);
  t3 := s3.AsCompressedString;
  SetOutput ( PChar (t3), buf2, size );
  s1.Destroy ();
  s2.Destroy ();
  s3.Destroy ();
  Result := 1;
end;

{$R *.res}

exports Union        name 'union';
exports Intersection name 'intersect';
exports Exclusion    name 'exclude';
exports Missing      name 'missing';

begin
end.
